package com.khmer9.lotto.remote.service;

import com.khmer9.lotto.remote.SocketMessageDataModel;

import org.json.JSONObject;

public interface EventListener {

    void onConnect();

    void onConnectionError() ;

    void onReconnect();

    void onReconnectionTimeout();

    void onDisconnect();

    void onNewLottoMessage(SocketMessageDataModel message);

    void onNewCreatedBettingMessage(JSONObject createdBettingJsonObject);

}
